#!/bin/bash
set -e

source config
LOG=${LOG_DIR}/datagen.log
> $LOG # clear the log

start=$(($(date +%s%N)/1000000))

# Compile
log info "Compiling source code ..." | tee -a $LOG 2>&1
{
  # Copy from offical toolset, apply patches, build dsdgen, build Hadoop job jar file
  mkdir -p ${BUILD_DIR}; mkdir -p ${DATAGEN_DIR}/target/lib
  rm -fr ${BUILD_DIR}/${TOOLS_VERSION}; cp -r $bin/${TOOLS_VERSION} ${BUILD_DIR}

  # apply patches
  cat $bin/patches/tools/*.diff | patch -p0 -d ${BUILD_DIR}/${TOOLS_VERSION}/tools

  cd ${BUILD_DIR}/${TOOLS_VERSION}/tools;make clean; make dsdgen 
  log info `./dsdgen -release 2>&1`

  cd ${BUILD_DIR}/${TOOLS_VERSION}; jar cvf dsdgen.jar tools/; cp dsdgen.jar ${DATAGEN_DIR}/target/lib
  cd ${DATAGEN_DIR}; mvn package
} >> $LOG 2>&1

# Generate text file on HDFS
hadoop fs -rm -r -f -skipTrash ${HDFS_DIR}/${SF}/ >> $LOG 2>&1

log info "Generating data ..." | tee -a $LOG 2>&1
cd ${DATAGEN_DIR}; hadoop jar target/*.jar -Ddfs.blocksize=1073741824 -Ddfs.replication=2 -d ${HDFS_DIR}/${SF}/ -s ${SF} -p ${DG_PARALLEL} >> $LOG 2>&1

end=$(($(date +%s%N)/1000000))
duration=$(( (end - start) / 1000))
sec=`bc <<< "scale=3; ($end - $start)/1000"`
elapse_time=`printf "%d:%02d:%02d, %s seconds" $(($duration / 3600)) $((($duration / 60) % 60)) $(($duration % 60)) $sec`
log info "Data generation finished. Time token: $elapse_time" | tee -a $LOG 2>&1

start=$(($(date +%s%N)/1000000))

log info "Generating maitenance data ..." | tee -a $LOG 2>&1
cd ${DATAGEN_DIR}; hadoop jar target/*.jar -d ${HDFS_DIR}/${SF}/mt -s ${SF} -u 4 >> $LOG 2>&1

log info  "Preparing maitenance data ..." | tee -a $LOG 2>&1
# Prepare the maintenance data
{
  tables="s_call_center s_catalog_order s_catalog_order_lineitem s_catalog_page s_catalog_returns s_customer s_customer_address s_inventory s_item s_promotion s_purchase s_purchase_lineitem s_store s_store_returns s_warehouse s_web_order s_web_order_lineitem s_web_page s_web_returns s_web_site s_zip_to_gmt"

  for s in $(seq 1 4); do
    hadoop fs -mkdir ${HDFS_DIR}/${SF}/mt_$s
    for t in $tables; do
      hadoop fs -mkdir ${HDFS_DIR}/${SF}/mt_$s/$t
      hadoop fs -mv ${HDFS_DIR}/${SF}/mt/$t/data-m*$(($s - 1)) ${HDFS_DIR}/${SF}/mt_$s/$t
    done
  done

  for t in $tables; do
    hadoop fs -rm -r ${HDFS_DIR}/${SF}/mt/$t > /dev/null 2>&1 # remove the empty folder
  done
} >> $LOG 2>&1

end=$(($(date +%s%N)/1000000))
duration=$(( (end - start) / 1000))
sec=`bc <<< "scale=3; ($end - $start)/1000"`
elapse_time=`printf "%d:%02d:%02d, %s seconds" $(($duration / 3600)) $((($duration / 60) % 60)) $(($duration % 60)) $sec`
log info "Maitenance data generation and preparation finished. Time token: $elapse_time" | tee -a $LOG 2>&1

start=$(($(date +%s%N)/1000000))

log info "Download maitenance input data from hdfs ..." | tee -a $LOG 2>&1
{
  if [[ -d ${BUILD_DIR}/delete ]]; then
    rm -rf ${BUILD_DIR}/delete
  fi
  if [[ -d ${BUILD_DIR}/inventory_delete ]]; then
    rm -rf ${BUILD_DIR}/inventory_delete
  fi
  hadoop fs -get ${HDFS_DIR}/${SF}/mt/delete ${BUILD_DIR}/
  hadoop fs -get ${HDFS_DIR}/${SF}/mt/inventory_delete ${BUILD_DIR}/
  cd ${BUILD_DIR}/delete
  i=1
  for file in $(ls ./); do
    mv $file delete_$i.dat
    i=$((i+1))
  done
  cd -
  cd ${BUILD_DIR}/inventory_delete
  i=1
  for file in $(ls ./); do
    mv $file inventory_delete_$i.dat
    i=$((i+1))
  done
  cd -
} >> $LOG 2>&1

end=$(($(date +%s%N)/1000000))
duration=$(( (end - start) / 1000))
sec=`bc <<< "scale=3; ($end - $start)/1000"`
elapse_time=`printf "%d:%02d:%02d, %s seconds" $(($duration / 3600)) $((($duration / 60) % 60)) $(($duration % 60)) $sec`
log info "Download maitenance data finished. Time token: $elapse_time" | tee -a $LOG 2>&1

