#!/bin/bash
set -e

source config
LOG=${LOG_DIR}/validate_data.log
> $LOG # clear the log

start=$(($(date +%s%N)/1000000))

VLD_DB=tpcds_vld_${SF}_${STORE}

DG_DIR=./build/${TOOLS_VERSION}/tools
VLD_DATA_DIR=/tmp/data

log info "Generating the validation data ..." | tee -a $LOG 2>&1
{
  if [[ -d ${VLD_DATA_DIR} ]]; then
    rm -rf ${VLD_DATA_DIR}
  fi
  mkdir -p ${VLD_DATA_DIR}
  cd ${DG_DIR};make clean; make dsdgen
  ./dsdgen  -scale ${SF} -validate -vcount 50  -dir ${VLD_DATA_DIR}
  cd -
} >> $LOG 2>&1

log info "Init the validation db ..." | tee -a $LOG 2>&1
beeline --showStartEndTime=true -u jdbc:hive2://localhost:${PORT} -f ./sqls/init_db.sql -hivevar DB=${VLD_DB} >> $LOG 2>&1

log info "Load data to validation db..." | tee -a $LOG 2>&1
beeline --showStartEndTime=true -u jdbc:hive2://localhost:${PORT} -f ./sqls/create_vld_tables.sql -hivevar DB=${VLD_DB} -hivevar VLD_DATA_DIR=${VLD_DATA_DIR} >> $LOG 2>&1

log info "Running Validate_Data.sql ..." | tee -a $LOG 2>&1
beeline --showStartEndTime=true -u jdbc:hive2://localhost:${PORT}/${DB} -f ./sqls/Validate_Data.sql -hivevar DB=${DB} -hivevar VLD_DB=${VLD_DB} >> $LOG 2>&1
results=`sed -n '/^|\ [0-9]/p' $LOG | awk '{print $2}' | xargs`
failed=0
for result in $results; do
  if [[ "$result" != "0" ]]; then
    failed=$(($failed+1))
  fi
done
log info "There's $failed failures in Validate_Data test." | tee -a $LOG 2>&1

log info "Running Check_RI.sql ..." | tee -a $LOG 2>&1
beeline --showStartEndTime=true -u jdbc:hive2://localhost:${PORT} -f ./sqls/Check_RI.sql -hivevar DB=${DB} > ${LOG_DIR}/Check_RI.log 2>&1
set +e
failed=`grep "| FAIL" ${LOG_DIR}/Check_RI.log | wc -l`
set -e
log info "There's $failed failures in Check_RI test." | tee -a $LOG 2>&1

log info "Running Check_Insert.sql ..." | tee -a $LOG 2>&1
beeline --showStartEndTime=true -u jdbc:hive2://localhost:${PORT} -f ./sqls/Check_Insert.sql -hivevar DB=${DB} > ${LOG_DIR}/Check_Insert.log 2>&1
log warn "We don't check the failures in Check_Insert test." | tee -a $LOG 2>&1

log info "Check cardinality of tables." | tee -a $LOG 2>&1

beeline --showStartEndTime=true -u jdbc:hive2://localhost:${PORT}/${DB} -f ./sqls/desc_tables.sql -hivevar DB=${DB}>$LOG_DIR/table_schema.out 2>>$LOG
beeline --showStartEndTime=true -u jdbc:hive2://localhost:${PORT}/${DB} -f ./sqls/count_tables.sql -hivevar DB=${DB}>$LOG_DIR/row_count.out 2>>$LOG

log info "Checking the refresh data." | tee -a $LOG 2>&1
for ii in `seq 1 4`; do
  sed -i "1s/\${DB}/${DB}/" ${BUILD_DIR}/mtsqls_$ii/count_mt_tables.sql
  log info "Count the row number of maitenance tables for round $ii." | tee -a $LOG 2>&1
  beeline --showStartEndTime=true -u jdbc:hive2://localhost:${PORT}/${DB} -f ${BUILD_DIR}/mtsqls_$ii/count_mt_tables.sql  -hivevar ROUND=$ii > $LOG_DIR/count_mt_tables_$ii.out 2>>$LOG
done

end=$(($(date +%s%N)/1000000))
duration=$(( (end - start) / 1000))
sec=`bc <<< "scale=3; ($end - $start)/1000"`
elapse_time=`printf "%d:%02d:%02d, %s seconds" $(($duration / 3600)) $((($duration / 60) % 60)) $(($duration % 60)) $sec`

log info "Validate data finished. Time token: $elapse_time" | tee -a $LOG 2>&1
